@@ -15,7 +15,9 @@ specific language governing permissions and limitations under the License.
 API 模块用于向外暴露接口，bamboo-engine 的使用者应该永远只用这个模块与 bamboo-engien 进行交互
 """
 
+import logging
 import functools
+import traceback
 from typing import Optional, Any
 from contextlib import contextmanager
 
@@ -23,6 +25,8 @@ from .utils.object import Representable
 from .eri import EngineRuntimeInterface
 from .engine import Engine
 
+logger = logging.getLogger("bamboo_engine")
+
 
 class EngineAPIResult(Representable):
     """
@@ -35,6 +39,7 @@ class EngineAPIResult(Representable):
         message: str,
         exc: Optional[Exception] = None,
         data: Optional[Any] = None,
+        exc_trace: Optional[str] = None,
     ):
         """
         :param result: 是否执行成功
@@ -50,6 +55,7 @@ class EngineAPIResult(Representable):
         self.message = message
         self.exc = exc
         self.data = data
+        self.exc_trace = exc_trace
 
 
 def _ensure_return_api_result(func):
@@ -58,17 +64,17 @@ def _ensure_return_api_result(func):
         try:
             data = func(*args, **kwargs)
         except Exception as e:
-            return EngineAPIResult(False, "fail", e, None)
+            logger.exception("{} raise error.".format(func.__name__))
+            trace = traceback.format_exc()
+            return EngineAPIResult(result=False, message="fail", exec=e, data=None, exc_trace=trace)
 
-        return EngineAPIResult(True, "success", None, data)
+        return EngineAPIResult(result=True, message="success", exc=None, data=data, exc_trace=None)
 
     return wrapper
 
 
 @_ensure_return_api_result
-def run_pipeline(
-    runtime: EngineRuntimeInterface, pipeline: dict, **options
-) -> EngineAPIResult:
+def run_pipeline(runtime: EngineRuntimeInterface, pipeline: dict, **options) -> EngineAPIResult:
     """
     执行 pipeline
 
@@ -84,9 +90,7 @@ def run_pipeline(
 
 
 @_ensure_return_api_result
-def pause_pipeline(
-    runtime: EngineRuntimeInterface, pipeline_id: str
-) -> EngineAPIResult:
+def pause_pipeline(runtime: EngineRuntimeInterface, pipeline_id: str) -> EngineAPIResult:
     """
     暂停 pipeline 的执行
 
@@ -102,9 +106,7 @@ def pause_pipeline(
 
 
 @_ensure_return_api_result
-def revoke_pipeline(
-    runtime: EngineRuntimeInterface, pipeline_id: str
-) -> EngineAPIResult:
+def revoke_pipeline(runtime: EngineRuntimeInterface, pipeline_id: str) -> EngineAPIResult:
     """
     撤销 pipeline，使其无法继续执行
 
@@ -119,9 +121,7 @@ def revoke_pipeline(
 
 
 @_ensure_return_api_result
-def resume_pipeline(
-    runtime: EngineRuntimeInterface, pipeline_id: str
-) -> EngineAPIResult:
+def resume_pipeline(runtime: EngineRuntimeInterface, pipeline_id: str) -> EngineAPIResult:
     """
     继续被 pause_pipeline 接口暂停的 pipeline 的执行
 
@@ -136,9 +136,7 @@ def resume_pipeline(
 
 
 @_ensure_return_api_result
-def pause_node_appoint(
-    runtime: EngineRuntimeInterface, node_id: str
-) -> EngineAPIResult:
+def pause_node_appoint(runtime: EngineRuntimeInterface, node_id: str) -> EngineAPIResult:
     """
     预约暂停某个节点的执行
 
@@ -153,9 +151,7 @@ def pause_node_appoint(
 
 
 @_ensure_return_api_result
-def resume_node_appoint(
-    runtime: EngineRuntimeInterface, node_id: str
-) -> EngineAPIResult:
+def resume_node_appoint(runtime: EngineRuntimeInterface, node_id: str) -> EngineAPIResult:
     """
     继续由于某个节点而暂停的 pipeline 的执行
 
@@ -170,9 +166,7 @@ def resume_node_appoint(
 
 
 @_ensure_return_api_result
-def retry_node(
-    runtime: EngineRuntimeInterface, node_id: str, data: Optional[dict] = None
-) -> EngineAPIResult:
+def retry_node(runtime: EngineRuntimeInterface, node_id: str, data: Optional[dict] = None) -> EngineAPIResult:
     """
     重试某个执行失败的节点
 
@@ -204,9 +198,7 @@ def skip_node(runtime: EngineRuntimeInterface, node_id: str) -> EngineAPIResult:
 
 
 @_ensure_return_api_result
-def skip_exclusive_gateway(
-    runtime: EngineRuntimeInterface, node_id: str, flow_id: str
-) -> EngineAPIResult:
+def skip_exclusive_gateway(runtime: EngineRuntimeInterface, node_id: str, flow_id: str) -> EngineAPIResult:
     """
     跳过某个执行失败的分支网关
 
@@ -223,9 +215,7 @@ def skip_exclusive_gateway(
 
 
 @_ensure_return_api_result
-def forced_fail_activity(
-    runtime: EngineRuntimeInterface, node_id: str, ex_data: str
-) -> EngineAPIResult:
+def forced_fail_activity(runtime: EngineRuntimeInterface, node_id: str, ex_data: str) -> EngineAPIResult:
     """
     强制失败某个 activity 节点
 
@@ -242,9 +232,7 @@ def forced_fail_activity(
 
 
 @_ensure_return_api_result
-def callback(
-    runtime: EngineRuntimeInterface, node_id: str, version: str, data: dict
-) -> EngineAPIResult:
+def callback(runtime: EngineRuntimeInterface, node_id: str, version: str, data: dict) -> EngineAPIResult:
     """
     回调某个节点
 
@@ -261,9 +249,7 @@ def callback(
 
 
 @_ensure_return_api_result
-def get_pipeline_states(
-    runtime: EngineRuntimeInterface, root_id: str, flat_children=True
-) -> EngineAPIResult:
+def get_pipeline_states(runtime: EngineRuntimeInterface, root_id: str, flat_children=True) -> EngineAPIResult:
     """
     返回某个任务的状态树
 
@@ -293,6 +279,8 @@ def get_pipeline_states(
                 "loop": s.loop,
                 "retry": s.retry,
                 "skip": s.skip,
+                "error_ignorable": s.error_ignored,
+                "error_ignored": s.error_ignored,
                 "created_time": s.created_time,
                 "started_time": s.started_time,
                 "archived_time": s.archived_time,
@@ -322,6 +310,8 @@ def get_pipeline_states(
         "loop": root_state.loop,
         "retry": root_state.retry,
         "skip": root_state.skip,
+        "error_ignorable": s.error_ignored,
+        "error_ignored": s.error_ignored,
         "created_time": root_state.created_time,
         "started_time": root_state.started_time,
         "archived_time": root_state.archived_time,
@@ -331,9 +321,7 @@ def get_pipeline_states(
 
 
 @_ensure_return_api_result
-def get_children_states(
-    runtime: EngineRuntimeInterface, node_id: str
-) -> EngineAPIResult:
+def get_children_states(runtime: EngineRuntimeInterface, node_id: str) -> EngineAPIResult:
     """
     返回某个节点及其所有子节点的状态
 
@@ -360,6 +348,8 @@ def get_children_states(
             "loop": s.loop,
             "retry": s.retry,
             "skip": s.skip,
+            "error_ignorable": s.error_ignored,
+            "error_ignored": s.error_ignored,
             "created_time": s.created_time,
             "started_time": s.started_time,
             "archived_time": s.archived_time,
@@ -376,6 +366,8 @@ def get_children_states(
         "loop": parent_state.loop,
         "retry": parent_state.retry,
         "skip": parent_state.skip,
+        "error_ignorable": parent_state.error_ignored,
+        "error_ignored": parent_state.error_ignored,
         "created_time": parent_state.created_time,
         "started_time": parent_state.started_time,
         "archived_time": parent_state.archived_time,
@@ -385,9 +377,7 @@ def get_children_states(
 
 
 @_ensure_return_api_result
-def get_execution_data_inputs(
-    runtime: EngineRuntimeInterface, node_id: str
-) -> EngineAPIResult:
+def get_execution_data_inputs(runtime: EngineRuntimeInterface, node_id: str) -> EngineAPIResult:
     """
     获取某个节点执行数据的输入数据
 
@@ -402,9 +392,7 @@ def get_execution_data_inputs(
 
 
 @_ensure_return_api_result
-def get_execution_data_outputs(
-    runtime: EngineRuntimeInterface, node_id: str
-) -> EngineAPIResult:
+def get_execution_data_outputs(runtime: EngineRuntimeInterface, node_id: str) -> EngineAPIResult:
     """
     获取某个节点的执行数据输出
 
@@ -419,9 +407,7 @@ def get_execution_data_outputs(
 
 
 @_ensure_return_api_result
-def get_execution_data(
-    runtime: EngineRuntimeInterface, node_id: str
-) -> EngineAPIResult:
+def get_execution_data(runtime: EngineRuntimeInterface, node_id: str) -> EngineAPIResult:
     """
     获取某个节点的执行数据
 
@@ -437,9 +423,7 @@ def get_execution_data(
 
 
 @_ensure_return_api_result
-def get_node_histories(
-    runtime: EngineRuntimeInterface, node_id: str, loop: int = -1
-) -> EngineAPIResult:
+def get_node_histories(runtime: EngineRuntimeInterface, node_id: str, loop: int = -1) -> EngineAPIResult:
     """
     获取某个节点的历史记录概览
 
@@ -469,9 +453,7 @@ def get_node_histories(
 
 
 @_ensure_return_api_result
-def get_node_short_histories(
-    runtime: EngineRuntimeInterface, node_id: str, loop: int = -1
-) -> EngineAPIResult:
+def get_node_short_histories(runtime: EngineRuntimeInterface, node_id: str, loop: int = -1) -> EngineAPIResult:
     """
     获取某个节点的简要历史记录
 
